@@ -17,9 +17,8 @@ our @ISA       = qw(Exporter);    ## no critic (ISA)
 our @EXPORT_OK = qw(
   bsearch_str       bsearch_str_pos     bsearch_str_range
   bsearch_num       bsearch_num_pos     bsearch_num_range
-  bsearch_general                       bsearch_general_range
-  bsearch_custom    
-  bsearch_transform 
+  bsearch_custom    bsearch_custom_pos
+  bsearch_transform
 );
 our %EXPORT_TAGS = ( all => \@EXPORT_OK );
 
@@ -29,7 +28,7 @@ our %EXPORT_TAGS = ( all => \@EXPORT_OK );
 
 ## no critic (prototypes)
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 # Needed for developer's releases: See perlmodstyle.
 # $VERSION = eval $VERSION;    ## no critic (eval,version)
@@ -85,44 +84,6 @@ sub bsearch_num ($\@) {
 }
 
 
-# Detect whether to search using string or numeric comparisons, then perform
-# the search.
-
-sub bsearch_general ($\@) {
-    my ( $target, $aref ) = @_;
-    my $min = 0;
-    my $max = $#{$aref};
-    if ( looks_like_number $target ) {
-        while ( $max > $min ) {
-            my $mid = int( ( $min + $max ) / 2 );
-            if ( $target > $aref->[$mid] ) {
-                $min = $mid + 1;
-            }
-            else {
-                $max = $mid;
-            }
-        }
-        return $min
-          if $max == $min && $target == $aref->[$min];
-    }
-    else {                                                  # Stringwise.
-        while ( $max > $min ) {
-            my $mid = int( ( $min + $max ) / 2 );
-            if ( $target gt $aref->[$mid] ) {
-                $min = $mid + 1;
-            }
-            else {
-                $max = $mid;
-            }
-        }
-        return $min
-          if $max == $min && $target eq $aref->[$min];
-    }
-    
-    return;
-}
-
-
 
 # Use a callback for comparisons.
 
@@ -195,7 +156,7 @@ sub bsearch_str_pos ($\@) {
     while ( $low < $high ) {
         use integer;
         my $cur = ( $low + $high ) / 2;
-        if ( $aref->[$cur] lt $target ) {
+        if ( $target gt $aref->[$cur] ) {
             $low = $cur + 1;    # too small, try higher
         }
         else {
@@ -214,7 +175,7 @@ sub bsearch_num_pos ($\@) {
     my ( $low, $high ) = ( 0, scalar @{$aref} );
     while ( $low < $high ) {
         my $cur = int( ( $low + $high ) / 2 );
-        if ( $aref->[$cur] < $target ) {
+        if ( $target > $aref->[$cur] ) {
             $low = $cur + 1;    # too small, try higher
         }
         else {
@@ -225,6 +186,24 @@ sub bsearch_num_pos ($\@) {
 }
 
 
+# Identical to bsearch_custom, but upon match-failure returns best insert
+# position for $target.
+
+sub bsearch_custom_pos (&$\@) {
+    my ( $comp, $target, $aref ) = @_;
+    my ( $low,    $high ) = ( 0, scalar @{$aref} );
+    while ( $low < $high ) {
+        my $cur = int( ( $low + $high ) / 2 );
+        if( $comp->( $target, $aref->[$cur] ) > 0 ) {
+            $low = $cur + 1;
+        }
+        else {
+            $high = $cur;
+        }
+    }
+    return $low;
+}
+
 
 # Given a low and a high target, returns a range of indices representing
 # where low and high fit into @haystack.
@@ -256,31 +235,6 @@ sub bsearch_num_range ($$\@) {
 }
 
 
-
-sub bsearch_general_range ($$\@) {
-    my ( $low_target, $high_target, $aref ) = @_;
-    my ( $index_low, $index_high );
-    if ( looks_like_number $low_target and looks_like_number $high_target ) {
-        $index_low  = bsearch_num_pos( $low_target,  @{$aref} );
-        $index_high = bsearch_num_pos( $high_target, @{$aref} );
-        if (   $index_high == @{$aref}
-            or $aref->[$index_high] > $high_target )
-        {
-            $index_high--;
-        }
-    }
-    else {
-        $index_low  = bsearch_str_pos( $low_target,  @{$aref} );
-        $index_high = bsearch_str_pos( $high_target, @{$aref} );
-        if (   $index_high == @{$aref}
-            or $aref->[$index_high] gt $high_target )
-        {
-            $index_high--;
-        }
-    }
-    return ( $index_low, $index_high );
-}
-
 1;    # End of List::BinarySearch
 
 __END__
@@ -291,11 +245,9 @@ List::BinarySearch - Binary Search a sorted list or array.
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
-Stable release.
-
-New functions: bsearch_num_pos, bsearch_str_pos, bsearch_general_range.
+New function: bsearch_custom_pos.
 
 
 =head1 SYNOPSIS
@@ -310,7 +262,6 @@ Examples:
     use List::BinarySearch qw(
         bsearch_str         bsearch_str_pos         bsearch_str_range
         bsearch_num         bsearch_num_pos         bsearch_num_range
-        bsearch_general                             bsearch_general_range
         bsearch_custom
         bsearch_transform
     );
@@ -323,7 +274,6 @@ Examples:
     # Find the first index of element containing the number 300.
     
     $index = bsearch_num       300, @num_array;
-    $index = bsearch_general   300, @num_array;
     $index = bsearch_custom    { $_[0] <=> $_[1] } 300, @num_array;
     $index = bsearch_transform { $_[0]           } 300, @num_array;
 
@@ -333,7 +283,6 @@ Examples:
     # Find the first index of element containing the string 'Mozart'.
 
     $index = bsearch_str       'Mozart', @str_array;
-    $index = bsearch_general   'Mozart', @str_array;
     $index = bsearch_custom    { $_[0] cmp $_[1] } 'Mozart', @str_array;
     $index = bsearch_transform { $_[0]           } 'Mozart', @str_array;
 
@@ -372,12 +321,9 @@ Examples:
 
     # The following functions return an inclusive range.
 
-    my( $low_ix, $high_ix )
-        = bsearch_general_range( 'Beethoven', 'Mozart', @str_array );
-        # Returns ( 1, 3 ), meaning ( 1 .. 3 ).
-
     my( $low_ix, $high_ix )
         = bsearch_str_range( 'Beethoven', 'Mozart', @str_array );
+        # Returns ( 1, 3 ), meaning ( 1 .. 3 ).
 
     my( $low_ix, $high_ix )
         = bsearch_num_range( 200, 400, @num_array );
@@ -459,15 +405,58 @@ finally, optimize.
 =head1 EXPORT
 
 Nothing is exported by default.  Upon request will export C<bsearch_str>,
-C<bsearch_num>, C<bsearch_general>, C<bsearch_custom>, C<bsearch_transform>,
-C<bsearch_num_pos>, C<bsearch_str_pos>, C<bsearch_str_range>,
-C<bsearch_num_range>, and C<bsearch_general_range>.  Or import all functions
+C<bsearch_num>, C<bsearch_custom>, C<bsearch_transform>,
+C<bsearch_num_pos>, C<bsearch_str_pos>, C<bsearch_custom_pos>,
+C<bsearch_str_range>, and C<bsearch_num_range>.  Or import all functions
 by specifying C<:all>.
 
 
 
 =head1 SUBROUTINES/METHODS
 
+=head2 WHICH SEARCH ROUTINE TO USE
+
+A binary search is supposed to be fast and efficient.  And it's such a good
+algorithm that in profiling this module it was observed that excessive logic
+paths and internal subroutine calls lead quickly to consuming more cycles
+than the algorithm itself for just about any data set that will fit into
+memory.  In the interest of keeping user interfaces as simple as possible, as
+well as limiting the overhead of complex decision paths and internal
+subroutine calls, this module presents a number of similar functions with
+subtle differences between them.  Here's a quick reference to which to choose:
+
+=over 4
+
+=item * C<bsearch_str>: Stringwise comparisons. Returns index or undef.
+
+=item * C<bsearch_str_pos>: Stringwise comparisons.  Returns index or index
+of insert point for needle.
+
+=item * C<bsearch_num>: Numeric comparisons.  Returns index or undef.
+
+=item * C<bsearch_num_pos>: Numeric comparisons.  Returns index or index of
+insert point for needle.
+
+=item * C<bsearch_custom>: Comparisons provided by user-defined callback.
+Returns index or undef.
+
+=item * C<bsearch_custom_pos>: Comparisons provided by user-defined callback.
+Returns index or index of insert point for needle.
+
+=item * C<bsearch_transform>: Transformations of list elements provided by
+user-defined callback.  Returns index or undef.
+
+=item * C<bsearch_str_range>: Stringwise comparisons for low and high needles.
+Returns a pair of indices refering to a range of elements corresponding to
+low and high needles.
+
+=item * C<bsearch_num_range>: Numeric comparisons for low and high needles.
+Returns a pair of indices referring to a range of elements corresponding to
+low and high needles.
+
+=back
+
+
 =head2 SUBROUTINE CATEGORIES
 
 There are three categories of subroutines.  Those that return undef (or an
@@ -478,13 +467,12 @@ C<$high_needle>.
 
 There are also several comparison styles, for use with different sorts of
 data.  The 'str' functions do stringwise comparisons.  The 'num' functions do
-numeric comparisons.  The 'general' functions detect whether C<$needle> is
-numeric or not.  If numeric, numeric comparisons are used.  If not, string
-comparisons are used.
+numeric comparisons.
 
 The 'custom' function uses a callback for the comparison.  The 'transform'
 function uses a callback to transform each list element in some user-defined
-way before doing a general comparison.
+way before doing comparisons that are either numeric or string depending on
+whether C<$needle> looks like a number or not.
 
 With that explanation, here are the functions:
 
@@ -512,22 +500,6 @@ The comparison type is numeric.
 
 
 
-=head2 bsearch_general NEEDLE ARRAY_HAYSTACK
-
-    $first_found_ix = bsearch_general $needle, @haystack;
-
-Detects whether C<$needle> is a string or number, and performs the
-appropriate comparisons to find C<$needle> in C<@haystack>.  Return
-value is an index to the first (lowest numbered) matching element in
-C<@haystack>.
-
-The comparison type is automatically detected for numbers or strings.  This
-extra magic is a convenience that does incur a small performance penalty.
-
-If C<$haystack> isn't found, the return value will be C<undef>.
-
-
-
 =head2 bsearch_custom CODE NEEDLE ARRAY_HAYSTACK
 
     $first_found_ix = bsearch_custom { $_[0] cmp $_[1] } $needle, @haystack;
@@ -577,7 +549,7 @@ C<bsearch_custom>.
 
 The only difference between this function and C<bsearch_str> is its return
 value upon failure.  C<bsearch_str> returns undef upon failure.
-C<bsearch_str_pos> returns the index of a valid insert point for C<$needle>
+C<bsearch_str_pos> returns the index of a valid insert point for C<$needle>.
 
 Finds the string specified by C<$needle> in the array C<@haystack>.  Return
 value is an index to the first (lowest numbered) matching element in
@@ -606,7 +578,7 @@ isn't found:
 
 The only difference between this function and C<bsearch_num> is its return
 value upon failure.  C<bsearch_num> returns undef upon failure.
-C<bsearch_num_pos> returns the index of a valid insert point for C<$needle>
+C<bsearch_num_pos> returns the index of a valid insert point for C<$needle>.
 
 Finds the string specified by C<$needle> in the array C<@haystack>.  Return
 value is an index to the first (lowest numbered) matching element in
@@ -628,26 +600,44 @@ C<@haystack> if it isn't found:
     }
 
 
+=head2 bsearch_custom_pos CODE NEEDLE ARRAY_HAYSTACK
+
+    $first_found_ix = bsearch_custom_pos { $_[0] cmp $_[1] } $needle, @haystack;
+    $first_found_ix = bsearch_custom_pos \&comparator,       $needle, @haystack;
+
+The only difference between this function and C<bsearch_custom> is its return
+value upon failure.  C<bsearch_custom> returns undef upon failure.
+C<bsearch_custom_pos> returns the index of a valid insert point for
+C<$needle>.
+
+Pass a code block or subref, a search target, and an array to search.  Uses
+the subroutine supplied in the code block or subref callback to test
+C<$needle> against elements in C<@haystack>.
+
+Return value is the index of the first element equaling C<$needle>.  If no
+element is found, the best insert-point for C<$needle> is returned.
+
+Beware a potential 'I<gotcha>': When dealing with complex data structures, the
+callback function will have an asymmetrical look to it, which is easy to
+get wrong.  The target will always be referred to by C<$_[0]>, but the right
+hand side of the comparison must refer to the C<$_[1]...>, where C<...> is
+the portion of the data structure to be used in the comparison: C<$_[1][$n]>,
+or C<$_[1]{$k}>, for example.
+
+
+
 =head2 bsearch_str_range LOW_STRING_NEEDLE HIGH_STRING_NEEDLE ARRAY_HAYSTACK
 
 =head2 bsearch_num_range LOW_NUMERIC_NEEDLE HIGH_NUMERIC_NEEDLE ARRAY_HAYSTACK
 
-=head2 bsearch_general_range LOW_NEEDLE HIGH_NEEDLE ARRAY_HAYSTACK
-
-    $first_found_ix = bsearch_general_range $low_needle, $high_needle, @haystack;
-
-Return a pair of indices that represent an inclusive range of elements from
-C<$low_needle> through C<$high_needle>.  Use string comparison, numeric
-comparison, or in the case of the 'general' function, automatically detect
-whether both targets are numeric, or not.  If both targets are numeric,
-numeric comparisons are used in the search.  Otherwise, string comparisons
-are used.
-
+Given C<$needle_low> and C<$needle_high>, return a low and high set of indices
+that represent the range of elements fitting within C<$needle_low> and
+C<$needle_high> (inclusive).
 
 Here's an example:
 
     my @haystack = ( 100, 200, 300, 400, 500 );
-    my( $low, $high ) = bsearch_general_range 200, 400, @haystack;
+    my( $low, $high ) = bsearch_num_range 200, 400, @haystack;
     my @found = @haystack[ $low .. $high ]; # @found holds ( 200, 300, 400 ).
 
 
@@ -747,9 +737,6 @@ function:
 
     sub transform { $_[0] }
 
-...but just use C<bsearch_general>, because that's what it does, without
-the callback.
-
 
 
 =head1 DATA SET REQUIREMENTS
@@ -770,7 +757,7 @@ makes sense if there are other good reasons for keeping the data set
 sorted in the first place.
 
 B<Passing an unsorted list to these Binary Search algorithms will result
-in undefined behavior.  There is validity checking.>
+in undefined behavior.  There is no validity checking.>
 
 A Binary Search consumes O(log n) time.  It would, therefore, be foolish
 for these algorithms to pre-check the list for sortedness, as that would
@@ -878,12 +865,12 @@ L<http://search.cpan.org/dist/List-BinarySearch/>
 
 =head1 ACKNOWLEDGEMENTS
 
-Thank-you to L<http://search.cpan.org/~corion/|Max Maischein> (Corion) for
+Thank-you to L<Max Maischein|http://search.cpan.org/~corion/> (Corion) for
 being a willing and helpful sounding board on API issues, and for spotting
 some POD problems.
 
-L<http://shop.oreilly.com/product/9781565923980.do|Mastering Algorithms with Perl>,
-from L<http://www.oreilly.com|O'Reilly>: for the inspiration (and much of the
+L<Mastering Algorithms with Perl|http://shop.oreilly.com/product/9781565923980.do>,
+from L<O'Reilly|http://www.oreilly.com>: for the inspiration (and much of the
 code) behind the positional and ranged searches.  Quoting MAwP: "I<...the
 binary search was first documented in 1946 but the first algorithm that worked
 for all sizes of array was not published until 1962.>" (A summary of a passage
